/***********************************************************************************
* Filename 		$RCSfile: alg1_board.c,v $
* Current CVS 		$Revision: 1.7 $
* Checked-in by 	$Author: yves $ 
*
* $Log: alg1_board.c,v $
* Revision 1.7  2005/03/28 22:32:38  yves
* Add typedef for position
*
* Revision 1.6  2004/12/18 13:49:50  yves
* correction on ep field, castling information corrected
*
* Revision 1.5  2004/12/18 11:53:10  yves
* add ep move
*
* Revision 1.4  2004/12/18 10:48:23  yves
* Move boardmapper and applymove to board source
*
* Revision 1.3  2004/12/17 17:29:39  yves
* Move function to here, copy data from standard board
*
* Revision 1.2  2004/12/16 00:18:24  yves
* Standard CVS command tags added
* FEN notation inverted, now uppercase is white and lowercase is black
*
*
***********************************************************************************/

#include "alg1_board.h"

// --------------------------------------------------------------------

t_alg1_board m1_board;
t_alg1_board m1_boardmapper;

// --------------------------------------------------------------------

extern struct	st_board m_board;

/**
 * copy 8x8 board into 144 array
 * and fill from user interface into engine data
 */
void search_initboard(void)
{
	int iRow, iColumn, i;
	posid iKingPos;
	
	// copy from ui board
	for (iRow=0;iRow<8;iRow++) {
		for (iColumn=0;iColumn<8;iColumn++) {
			m1_board.m_fields[iColumn+2+(iRow+2)*12] = m_board.m_fields[iColumn][iRow];
		}
	}
	
	// speedup evaluation later one
	for (iKingPos=STARTPOS;iKingPos<ENDPOS;iKingPos++) {
		if (m1_board.m_fields[iKingPos]==PIECE_KING) {
			m1_board.m_whiteking = iKingPos;
			break;
		}
	}
	
	for (iKingPos=STARTPOS;iKingPos<ENDPOS;iKingPos++) {
		if (m1_board.m_fields[iKingPos]==BLACK_KING) {
			m1_board.m_blackking = iKingPos;
			break;
		}
	}
	
	// set castle flag to allowed	
	m1_board.m_whitecastle[CASTLE_LONG] = myFALSE;
	m1_board.m_whitecastle[CASTLE_SHORT] = myFALSE;
	m1_board.m_blackcastle[CASTLE_LONG] = myFALSE;
	m1_board.m_blackcastle[CASTLE_SHORT] = myFALSE;
	
	// now we will base ourselves on the positions of the pieces
	if ( (m1_board.m_whiteking == KINGWHITESTART) && (m1_board.m_fields[26] == PIECE_ROOK) ) {
		m1_board.m_whitecastle[CASTLE_LONG] = myTRUE;
	}
	if ( (m1_board.m_whiteking == KINGWHITESTART) && (m1_board.m_fields[33] == PIECE_ROOK) ) {
		m1_board.m_whitecastle[CASTLE_SHORT] = myTRUE;
	}
	if ( (m1_board.m_blackking == KINGBLACKSTART) && (m1_board.m_fields[110] == BLACK_ROOK) ) {
		m1_board.m_blackcastle[CASTLE_LONG] = myTRUE;
	}
	if ( (m1_board.m_blackking == KINGBLACKSTART) && (m1_board.m_fields[117] == BLACK_ROOK) ) {
		m1_board.m_blackcastle[CASTLE_SHORT] = myTRUE;
	}
	
	// now we correct with the rook positions from the fen string
	if (m_board.m_piecemoved[COLOUR_WHITE][0]) {
		m1_board.m_whitecastle[CASTLE_LONG] = myFALSE;
	}
	if (m_board.m_piecemoved[COLOUR_WHITE][1]) {
		m1_board.m_whitecastle[CASTLE_SHORT] = myFALSE;
	}
	if (m_board.m_piecemoved[COLOUR_BLACK][0]) {
		m1_board.m_blackcastle[CASTLE_LONG] = myFALSE;
	}
	if (m_board.m_piecemoved[COLOUR_BLACK][1]) {
		m1_board.m_blackcastle[CASTLE_SHORT] = myFALSE;
	}
	
	// to keep scores, castling allowed or done increases the score
	m1_board.m_whitecastlescore = ( m1_board.m_whitecastle[CASTLE_LONG] + m1_board.m_whitecastle[CASTLE_SHORT] );
	m1_board.m_blackcastlescore = ( m1_board.m_blackcastle[CASTLE_LONG] + m1_board.m_blackcastle[CASTLE_SHORT] );
	
	// carry over ep data from ui board
	m1_board.m_epfield = m_board.m_epfield;
	
	// store flags for begin position
	// todo: correct because we will not always start at begin
	for (i=0;i<8;i++) {
		m1_board.m_whitebegin[i] = 1;
		m1_board.m_blackbegin[i] = 1;
	}
	
}

/**
 * pefrom the move no the board
 * and update board statistics
 */
void apply_movewhite(const posid iFrom, const posid iTo, const t_promotion cFlag)
{
	// handling for ep
	m1_board.m_epfield = 0;
	if (m1_board.m_fields[iFrom] == PIECE_PAWN) {
		if ((iTo-iFrom)==24) {
			// set epfield for next move from this position
			m1_board.m_epfield = iTo-12;
		} else {
			// test if this is an ep move (not a double and no real capture)
			if ((m1_board.m_fields[iTo] == FIELD_EMPTY)	&& ((iTo-iFrom)!=12)) {
				// clear out captured piece
				m1_board.m_fields[iTo-12] = FIELD_EMPTY;
			}
		}
	}

	// apply move
	if (cFlag != PROMOTION_NONE) {
		// promotion
		m1_board.m_fields[iTo] = toupper(cFlag) ;
	} else {
		m1_board.m_fields[iTo] = m1_board.m_fields[iFrom];
	}
	m1_board.m_fields[iFrom] = FIELD_EMPTY;
	
	// update king castling flags and do castling
	if (iFrom == m1_board.m_whiteking) {
		m1_board.m_whiteking = iTo; 
		if ((iTo-iFrom) == 2) {
			// short castle
			m1_board.m_fields[iTo-1] = m1_board.m_fields[iTo+1];
			m1_board.m_fields[iTo+1] = FIELD_EMPTY;
			// score flag
			m1_board.m_whitecastlescore = 8;  
			// begin pos for rook cleared
			m1_board.m_whitebegin[COL144TO64(iTo+1)] = 0;
		} else if ((iTo-iFrom)== -2) {
			// long castle
			m1_board.m_fields[iTo+1] = m1_board.m_fields[iTo-2];
			m1_board.m_fields[iTo-2] = FIELD_EMPTY;
			// score flag
			m1_board.m_whitecastlescore = 8; 
			// begin pos for rook cleared
			m1_board.m_whitebegin[COL144TO64(iTo-2)] = 0;
		} else {
			m1_board.m_whitecastlescore = 0;  
		}
		m1_board.m_whitecastle[CASTLE_LONG] = myFALSE;
		m1_board.m_whitecastle[CASTLE_SHORT] = myFALSE;		
	}

	// clear begin flag
	if (ROW144TO64(iFrom)==0) {
		m1_board.m_whitebegin[COL144TO64(iFrom)] = 0;
		
		if (COL144TO64(iFrom)==0) m1_board.m_whitecastle[CASTLE_LONG] = myFALSE;
		if (COL144TO64(iFrom)==7) m1_board.m_whitecastle[CASTLE_SHORT] = myFALSE;
	}
	
	// capture, so clear other flag
	if (ROW144TO64(iTo)==7) {
		m1_board.m_blackbegin[COL144TO64(iTo)] = 0;
		
		if (COL144TO64(iFrom)==0) m1_board.m_blackcastle[CASTLE_LONG] = myFALSE;
		if (COL144TO64(iFrom)==7) m1_board.m_blackcastle[CASTLE_SHORT] = myFALSE;
	}
}

// --------------------------------------------------------------------

void apply_moveblack(const posid iFrom, const posid iTo, const t_promotion cFlag)
{
	// handling for ep
	m1_board.m_epfield = 0;
	if (m1_board.m_fields[iFrom] == BLACK_PAWN) {
		if ((iFrom-iTo)==24) {
			// set epfield for next move from this position
			m1_board.m_epfield = iTo+12;
		} else {
			// test if this is an ep move (not a double and no real capture)
			if ((m1_board.m_fields[iTo] == FIELD_EMPTY)	&& ((iFrom-iTo)!=12)) {
				// clear out captured piece
				m1_board.m_fields[iTo+12] = FIELD_EMPTY;
			}
		}
	}

	// apply move
	if (cFlag != PROMOTION_NONE) {
		// promotion
		m1_board.m_fields[iTo] = tolower(cFlag);
	} else {
		m1_board.m_fields[iTo] = m1_board.m_fields[iFrom];
	}
	m1_board.m_fields[iFrom] = FIELD_EMPTY;
	
	// update king castling flags and do castling
	if (iFrom == m1_board.m_blackking) {
		m1_board.m_blackking = iTo; 
		if ((iTo-iFrom) == 2) {
			// short castle
			m1_board.m_fields[iTo-1] = m1_board.m_fields[iTo+1];
			m1_board.m_fields[iTo+1] = FIELD_EMPTY;
			// score flag
			m1_board.m_blackcastlescore = 8; 
			// begin pos for rook cleared
			m1_board.m_blackbegin[COL144TO64(iTo+1)] = 0;
		} else if ((iTo-iFrom)== -2) {
			// long castle
			m1_board.m_fields[iTo+1] = m1_board.m_fields[iTo-2];
			m1_board.m_fields[iTo-2] = FIELD_EMPTY;
			// score flag
			m1_board.m_blackcastlescore = 8; 
			// begin pos for rook cleared
			m1_board.m_blackbegin[COL144TO64(iTo-2)] = 0;
		} else {
			m1_board.m_blackcastlescore = 0; 
		}
		m1_board.m_blackcastle[CASTLE_LONG] = myFALSE;
		m1_board.m_blackcastle[CASTLE_SHORT] = myFALSE;
		
	}

	// clear begin flag
	if (ROW144TO64(iFrom)==7) {
		m1_board.m_blackbegin[COL144TO64(iFrom)] = 0;
		
		if (COL144TO64(iFrom)==0) m1_board.m_blackcastle[CASTLE_LONG] = myFALSE;
		if (COL144TO64(iFrom)==7) m1_board.m_blackcastle[CASTLE_SHORT] = myFALSE;
	}
	
	// capture, so clear other flag
	if (ROW144TO64(iTo)==0) {
		m1_board.m_whitebegin[COL144TO64(iTo)] = 0;
		
		if (COL144TO64(iFrom)==0) m1_board.m_whitecastle[CASTLE_LONG] = myFALSE;
		if (COL144TO64(iFrom)==7) m1_board.m_whitecastle[CASTLE_SHORT] = myFALSE;
	}	
}
