/***********************************************************************************
* Filename 		$RCSfile: alg1_board.h,v $
* Current CVS 		$Revision: 1.10 $
* Checked-in by 	$Author: yves $ 
*
* $Log: alg1_board.h,v $
* Revision 1.10  2005/03/28 22:27:02  yves
* Add typedef for position
*
* Revision 1.9  2004/12/24 01:25:34  yves
* Opening book in binary mode implemented.
* Version 0.2.6
*
* Revision 1.6  2004/12/18 11:53:10  yves
* add ep move
*
* Revision 1.5  2004/12/18 10:48:23  yves
* Move boardmapper and applymove to board source
*
* Revision 1.4  2004/12/17 17:35:30  yves
* copy from search
*
* Revision 1.3  2004/12/16 03:02:07  yves
* Change to bitfields
*
* Revision 1.2  2004/12/16 00:18:25  yves
* Standard CVS command tags added
* FEN notation inverted, now uppercase is white and lowercase is black
*
*
***********************************************************************************/

#if !defined(__ALG1_BOARD_H__)
#define __ALG1_BOARD_H__

#include "belofte.h"
#include "board.h"

/*
 *			a	b	c	d	e	f	g	h
 *	132 133 134 135 136 137 138 139 140 141 142 143
 *	120
 *	108	109	110	111	112	113	114	115	116	117	118	119		- row 8
 *	96  97  98  99  100 101 102 103 104 105 106 107     - row 7
 *	84
 *	72
 *	60
 *	48
 *	36  37  38  39  40  41  42  43  44  45  46  47      - row 2
 *	24	25	26	27	28	29	30	31	32	33	34	35		- row 1
 *	12	13	14	15	16	17	18	19	20	21	22	23
 *	0	1	2	3	4	5	6	7	8	9	10	11
 *			a	b	c	d	e	f	g	h
 */

// positions where the board itself starts and ends
#define STARTPOS 26
#define ENDPOS   118
#define KINGWHITESTART 30
#define KINGBLACKSTART 114

#define BOARDARRAY_SIZE 144

// --------------------------------------------------------------------

#define M1_LIJN(x) ((x) % 8)
#define M1_RIJ(x) ((x) / 8)

// --------------------------------------------------------------------

#define MAP144TO64(x) (m1_boardmapper.m_fields[(x)])
#define ROW144TO64(x) (m1_boardmapper.m_fields[(x)] / 8)
#define COL144TO64(x) (m1_boardmapper.m_fields[(x)] % 8)

// --------------------------------------------------------------------

struct st_alg1_board {
	t_field m_fields[BOARDARRAY_SIZE];	// fields as described above
	posid m_whiteking;				// king position
	posid m_blackking;	
	byte m_whitecastlescore;		// score data for improving evaluation
	byte m_blackcastlescore;
	byte m_whitecastle[2];			// short and long castle possible
	byte m_blackcastle[2];			
	byte m_whitebegin[8];			// pieces on begin position
	byte m_blackbegin[8];			
	posid m_epfield;				// field where ep is possible, 0 if none
	byte m_rule50;					// number of plies since last capture or pawn move
};

// --------------------------------------------------------------------

enum tCastleFlags { CASTLE_LONG = 0, CASTLE_SHORT };

// --------------------------------------------------------------------

typedef struct st_alg1_board	t_alg1_board;

// --------------------------------------------------------------------

void	apply_movewhite(const posid iFrom, const posid iTo, const t_promotion cFlag);
void	apply_moveblack(const posid iFrom, const posid iTo, const t_promotion cFlag);

// --------------------------------------------------------------------

#endif
