/*----------------------------------------------------------------------+
|                                                                      |
|              belofte.c - A Promising Chess Program                   |
|                                                                      |
+----------------------------------------------------------------------+
|    @author Yves Debilloez                                            |
|    @since 01-oct-2004                                                |
+----------------------------------------------------------------------+
|    Copyright (C)2004-2004 Yves Debilloez                             |
|    This program is distributed under the GNU General Public License. |
+----------------------------------------------------------------------*/

/**
* The main program, doing all coordnation stuff in between the rest
 * @see usercmd.c and board.c
 */

/***********************************************************************************
* Filename 		$RCSfile: belofte.c,v $
* Current CVS 	$Revision: 1.7 $
* Checked-in by $Author: yves $ 
*
* $Log: belofte.c,v $
* Revision 1.7  2005/03/28 22:34:26  yves
* Change default depth
*
* Revision 1.6  2004/12/17 23:36:22  yves
* default depth back to 3 because of bad play
*
* Revision 1.5  2004/12/17 17:37:54  yves
* search depth default to 4
*
* Revision 1.4  2004/12/16 00:18:24  yves
* Standard CVS command tags added
* FEN notation inverted, now uppercase is white and lowercase is black
*
*
***********************************************************************************/

#include "belofte.h"
#include "alg1_search.h"

#define DEFAULT_DEPTH 2

/*----------------------------------------------------------------------+
 |      data structures                                                 |
 +----------------------------------------------------------------------*/

extern int algoritme;
extern t_depth maxdepth;
extern int computer_engines[2];
extern int force_mode;
extern int xboard_mode;
extern int curply;

/**
* Main entry point
 * <br>contains main loop
 * @param argc The number of parameters
 * @param argv The strings pointing to each parameter
 */

int main(int argc, char *argv[])
{	
	char szMove[20];
	
	if (argc>1) {
		// do not print out anything if the application gets booted with an xboard parameter as athe
		// first parameter
		if (strcmp(argv[1],"xboard") == 0)
			xboard_mode = 1;
		else
			xboard_mode = 0;
	}
	
	search_constructor();
	cmd_constructor("","");
	maxdepth = DEFAULT_DEPTH;
	srand(time(0));
	
	for (;;) { 
			
		cmd_readcommand();
		// cmd_go("","");  // used for profiling
		
		while (1) {
			
			if ((!force_mode) && (computer_engines[curply%2])) {
				// depending on the algorithm set, use the correct search
				if (algoritme==0) {
					search_root(maxdepth,szMove,sizeof(szMove));
					cmd_snd_move(szMove);
				} else if (algoritme==1) {	
					// move = search_yvesvar1(maxdepth);
				} else {
					// move = search_yvesvar2(maxdepth);
				}
			} else 
				// there is no computer engine that plays, so return to user
				break;
			
			// in between each computer move, we print the board
			if (!xboard_mode) {
				board_print();
			}
			
			if (cmd_peek()) break;
		}
	}
	return 0;
}


/*----------------------------------------------------------------------+
 |                                                                      |
 +----------------------------------------------------------------------*/

