/***********************************************************************************
* Filename 		$RCSfile: belofte.h,v $
* Current CVS 		$Revision: 1.13 $
* Checked-in by 	$Author: yves $ 
*
* $Log: belofte.h,v $
* Revision 1.13  2005/03/28 22:25:05  yves
* Add typedef for position
*
* Revision 1.12  2005/03/20 15:11:42  yves
* Changes before sending to CH, no checking
*
* Revision 1.11  2004/12/24 01:25:34  yves
* Opening book in binary mode implemented.
* Version 0.2.6
*
* opening book implemented
* version number increase
* When double moves exist, make sure to write the correct move
* add max function
* add defines
*
* Revision 1.4  2004/12/16 00:18:24  yves
* Standard CVS command tags added
* FEN notation inverted, now uppercase is white and lowercase is black
*
***********************************************************************************/

#if !defined(__BELOFTE_H__)
#define __BELOFTE_H__

#include <ctype.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

// for srand(time(0)) call
#include <time.h>  

// --------------------------------------------------------------------

#if defined(__i386) && !defined(__LITTLE_ENDIAN__)
#define __LITTLE_ENDIAN__ 1
#endif

#if defined(_MSC_VER) && !defined(__LITTLE_ENDIAN__) && !defined(__BIG_ENDIAN__)
#define __LITTLE_ENDIAN__ 1
#endif

#if !defined(__BIG_ENDIAN__) && !defined(__LITTLE_ENDIAN__)
#error Do not know the endianess of this architecture
#endif

// --------------------------------------------------------------------

#define MYNAME "Belofte"
#define MYVERSION "0.2.8"

#define BOOKNAME	"belofte-" MYVERSION ".book"
#define PGNNAME     "belofte-" MYVERSION ".pgn"
#define LOGNAME     "belofte-" MYVERSION ".log"

#define MAX_DATANAME_LENGTH 256
#define MAX_FEN_LENGTH 128
#define MAX_GAME_LENGTH 256
#define MAX_MOVES_PER_POSITION 200

// 0-0-0, e4xd3ep, e7xf8D+, 
#define MAX_MOVE_LENGTH    8

// --------------------------------------------------------------------

enum tBoolean { myFALSE = 0, myTRUE };
enum tFillMoves { NO_FILLMOVES = 0, FILLMOVES };
enum tColour { COLOUR_WHITE = 0, COLOUR_BLACK };

typedef int					t_field;
typedef int					t_flag;
typedef enum tColour		t_colour;
typedef int					t_ply;
typedef unsigned char		byte;
typedef unsigned short int	posid;
typedef enum tBoolean		t_boolean;
typedef float				t_score;
typedef float				t_depth;

// --------------------------------------------------------------------

void board_setup(char *szFen);
void board_print(void);
int  board_applymove(const char *szCoordMove);
int  board_unapplymove(void);

void cmd_constructor(char *szCommand, char *szArg);
void cmd_constructor2(void);
void cmd_new(char *szCommand, char *szArg);
void cmd_go(char * szCommand, char * szArg);  // for profiling
void cmd_sd(char * szCommand, char * szArg);
void cmd_force(char * szCommand, char * szArg);

int  cmd_readcommand(void);
int  cmd_peek(void);
int  cmd_readline(char * szBuffer, void *pFile, const size_t nSize);
int  file_readline(char * szBuffer, FILE *pFile, const size_t nSize);

void cmd_snd(const char *szMessage);
void cmd_snd_info(const char *szMessage);
void cmd_snd_move(const char *szMove);
void cmd_snd_thinking(const int nPly, const t_score fEval, const char *szThinking);

// --------------------------------------------------------------------

enum { 
	A1=0, B1, C1, D1, E1, F1, G1, H1 ,
	A2, B2, C2, D2, E2, F2, G2, H2 ,
	A3, B3, C3, D3, E3, F3, G3, H3 ,
	A4, B4, C4, D4, E4, F4, G4, H4 ,
	A5, B5, C5, D5, E5, F5, G5, H5 ,
	A6, B6, C6, D6, E6, F6, G6, H6 ,
	A7, B7, C7, D7, E7, F7, G7, H7 ,
	A8, B8, C8, D8, E8, F8, G8, H8 
} ;

// --------------------------------------------------------------------

#define MOVERESULT_OK    0
#define MOVERESULT_NIL   -1
#define MOVERESULT_ERROR -99

// --------------------------------------------------------------------

#if !defined(min)
#define min(a,b) ((a)<(b)?(a):(b))
#endif

#if !defined(max)
#define max(a,b) ((a)>(b)?(a):(b))
#endif

// --------------------------------------------------------------------

#endif // def __YVESCHESS_